import "./env";
import "./database";
import cors from "cors";
import Raven from "raven";
import helmet from "helmet";
import morgan from "morgan";
import express from "express";
import routes from "./routes";
import bodyParser from "body-parser";
import compression from "compression";
import json from "./middlewares/json";
import logger, {logStream} from "./utils/logger";
import * as errorHandler from "./middlewares/errorHandler";
import packageJson from "../package";

function getAppInfos() {
    const APP_PORT = (process.env.NODE_ENV === "test" ? process.env.TEST_APP_PORT : process.env.APP_PORT) || process.env.PORT || "3000";
    const APP_HOST = process.env.APP_HOST || "0.0.0.0";
    const DOMAIN = process.env.DOMAIN || "http:localhost";

    return {
        port: APP_PORT,
        host: APP_HOST,
        domain: DOMAIN,
        title: packageJson.name,
        version: packageJson.version,
        description: packageJson.description,
        author: packageJson.author
    }
}

function registerAppInfo(app, info) {
    app.set("host", info.host);
    app.set("port", info.port);

    app.locals.title = info.name;
    app.locals.version = info.version;
    app.locals.description = info.description;
    app.locals.author = info.author;
}

function registerRoutes(app) {
    app.use("/api", routes);
}

function registerApps(app, info) {
    Raven.config(process.env.SENTRY_DSN).install();

    // This request handler must be the first middleware on the app
    app.use(Raven.requestHandler());

    app.use(cors({
        origin: [info.domain],
        methods: ["GET", "POST", "PUT", "DELETE"],
        credentials: true
    }));
    app.use(helmet());
    app.use(compression());
    app.use(morgan("tiny", {stream: logStream}));
    app.use(bodyParser.json());
    app.use(errorHandler.bodyParser);
    app.use(json);
}

function registerMiddlewares(app) {
    app.use(Raven.errorHandler());
    app.use(errorHandler.genericErrorHandler);
    app.use(errorHandler.methodNotAllowed);
}

function pringBanner() {
    const banner = ':::\'###::::\'########::\'########::::::\'######::\'########::::\'###::::\'########::\'########:\'########:\'########::\n' +
        '::\'## ##::: ##.... ##: ##.... ##::::\'##... ##:... ##..::::\'## ##::: ##.... ##:... ##..:: ##.....:: ##.... ##:\n' +
        ':\'##:. ##:: ##:::: ##: ##:::: ##:::: ##:::..::::: ##:::::\'##:. ##:: ##:::: ##:::: ##:::: ##::::::: ##:::: ##:\n' +
        '\'##:::. ##: ########:: ########:::::. ######::::: ##::::\'##:::. ##: ########::::: ##:::: ######::: ##:::: ##:\n' +
        ' #########: ##.....::: ##.....:::::::..... ##:::: ##:::: #########: ##.. ##:::::: ##:::: ##...:::: ##:::: ##:\n' +
        ' ##.... ##: ##:::::::: ##:::::::::::\'##::: ##:::: ##:::: ##.... ##: ##::. ##::::: ##:::: ##::::::: ##:::: ##:\n' +
        ' ##:::: ##: ##:::::::: ##:::::::::::. ######::::: ##:::: ##:::: ##: ##:::. ##:::: ##:::: ########: ########::\n' +
        '..:::::..::..:::::::::..:::::::::::::......::::::..:::::..:::::..::..:::::..:::::..:::::........::........:::';
    logger.info(banner);
}

function buildApp() {
    const app = express();

    const appInfo = getAppInfos();
    registerAppInfo(app, appInfo);
    registerApps(app, appInfo);
    registerRoutes(app);
    registerMiddlewares(app);

    return app;
}

function startApp(app) {
    app.listen(app.get("port"), app.get("host"), () => {
        pringBanner();
        logger.info(`Server started at http://${app.get("host")}:${app.get("port")}/api`);
    });
}

export default {buildApp, startApp};
